// PIC32MX150F/250F用SDブートローダー　バージョン2対応　情報記録プログラム
//　フラッシュの固定領域に共通形式で記録することで、1つのボードで複数のアプリのハイスコアなどの情報を保存可能
//　形式：ファイル名8バイト＋32bit×2個の情報で1レコード。最大63レコードを保持可能

// 0x9d005800-0x9d005bfff:記録領域
// 0x9d005c00-0x9d005ffff:ワーク領域（ページ消去すると他のアプリ用データまで消えるので、最初にここに退避する）
// （RAM容量が少ないのでフラッシュに一時退避する）

#include <plib.h>
#include "recscore.h"
void *recscore_ad;//レコードへのポインタ、recscore_init()で設定される

// 記録領域からワーク領域にコピーし、ロードしたファイル名の先頭8文字でレコードを検索。なければ新規レコードを確保
// グローバル変数recscore_adにレコード先頭アドレスを設定
// 記録領域がいっぱいで作成できなければrecscore_ad=0を返す
void init_recscore(){
	unsigned int *ps,*pd;
	unsigned int id1,id2;

	recscore_ad=0;
	id1=*((unsigned int *)RECSCOREFLASH);
	id2=*((unsigned int *)(RECSCOREFLASH+4));
	ps=(unsigned int *)(RECSCOREFLASH+16);
	while(ps<(unsigned int *)(RECSCOREFLASH+RECSCORESIZE)){
		if(*ps==0xffffffff || *ps==id1 && *(ps+1)==id2){
			recscore_ad=ps;
			break;
		}
		ps+=4;
	}
	if(recscore_ad==0) return;
	NVMErasePage((void *)RECSCOREFLASHCOPY);//ページ消去
	ps=(unsigned int *)RECSCOREFLASH;
	pd=(unsigned int *)RECSCOREFLASHCOPY;
	while(ps<(unsigned int *)(RECSCOREFLASH+RECSCORESIZE)){
		if(*ps==0xffffffff){
			if(ps==(unsigned int *)recscore_ad){
				NVMWriteWord(pd++,id1);
				NVMWriteWord(pd++,id2);
				NVMWriteWord(pd++,0);
				NVMWriteWord(pd  ,0);
			}
			return;
		}
		NVMWriteWord(pd++,*ps++);
		NVMWriteWord(pd++,*ps++);
		NVMWriteWord(pd++,*ps++);
		NVMWriteWord(pd++,*ps++);
	}
}

// 2つの32ビットデータを書き込み
void write_recscore(unsigned int s1,unsigned int s2){
	unsigned int *ps,*pd;

	if(recscore_ad==0) return;// 書き込み領域なし
	NVMErasePage((void *)RECSCOREFLASH);//ページ消去
	ps=(void *)RECSCOREFLASHCOPY;
	pd=(void *)RECSCOREFLASH;
	while(pd<(unsigned int *)(RECSCOREFLASH+RECSCORESIZE)){
		if(*ps==0xffffffff) return;
		if(pd!=(unsigned int *)recscore_ad){
			NVMWriteWord(pd++,*ps++);//ワード書き込み
			NVMWriteWord(pd++,*ps++);
			NVMWriteWord(pd++,*ps++);
			NVMWriteWord(pd++,*ps++);
		}
		else{
			NVMWriteWord(pd++,*ps++);
			NVMWriteWord(pd++,*ps++);
			NVMWriteWord(pd++,s1);
			NVMWriteWord(pd++,s2);
			ps+=2;
		}
	}
}

// レコードのn番目（0～1）の32ビットデータを返す
unsigned int read_recscore(int n){
	if(recscore_ad==0) return 0;
	return *((unsigned int *)recscore_ad+n+2);
}
